#ifndef __OctreeCHCOcclusion_H
#define __OctreeCHCOcclusion_H


#include "OctreeCHCPrerequisites.h"

#include <stack>
#include <list>

#include "OctreeCHCOcclusionQuerySet.h"
#include "OctreeCHCOcclusionElement.h"

#include "OctreeCHC.h"
#include "OctreeCHCNode.h"

namespace OgreOctreeCHC
{

    class OcclusionCulling 
    {

        public:
            OcclusionCulling(unsigned  int visibilityThsd = 0);
            ~OcclusionCulling()
            {
                mQueryPool.deletePool ();
            }
	        //this is the algorithm from the paper
	        void CHCdepthTraversal(SpatialTree *tree, Ogre::VisibleObjectsBoundsInfo * const visibleBounds);
            void CHCdepthTraversalConservative (SpatialTree *tree, Ogre::VisibleObjectsBoundsInfo * const visibleBounds);
  
	        bool insideViewFrustum (OcclusionElement& node);
            void pullUpVisibility(OcclusionElement& n);
            bool isQueryResultIsVisible(OcclusionElement& node);    
            bool issueDrawQuery(SpatialTreeNode& n);
            bool issueBboxQuery(OcclusionElement& n);
            void queueDraw(SpatialTreeNode& n);

	        const unsigned int getNodeCount() const { return mNodeCount; }
            unsigned int getFrame() const {return mFrameId;};
            OctreeCHCCamera *getCamera() const {return mCurrentCam;};

           
        	
	            unsigned int mObjectCount;
	            unsigned int mTriangleCount;
	            unsigned int mTraversedNodesCount;
	            unsigned int mFrustumCulledNodesCount;
	            unsigned int mQueryCount;
        	 

            bool nextFrame (OctreeCHCCamera *cam, 
                            bool onlyshadowcaster,
							Ogre::RenderQueue *q);

            void init (OctreeCHCSceneManager *scnMngr)
            {
                mScnMngr = scnMngr;
            };

            void initQueryPool()
            {
                if (mIsQueryPoolNotInitiated)
                {   
                    mQueryPool.setRenderSystem ();
                    mQueryPool.setAutoextend (true);
                    mQueryPool.setPoolSize (300);
                    mIsQueryPoolNotInitiated = false;
                }
            };

               
            void TreeOverlayDebugView(SpatialTree *octree, 
                                        OctreeCHCSceneManager *mScnMgr);
        

            OctreeCHCSceneManager *mScnMngr;

	        OctreeCHCCamera *mCurrentCam;
			Ogre::RenderQueue *mCurrentRenderQueue;
            bool mOnlyShadowCaster;
	        unsigned int mFrameId;
			bool mDisplayNode;
			bool mShowBoxes;

			void setIfNotSolidScene(const bool isNotSolid) {mIsNotSolidScene = isNotSolid;};
			bool isNotSolidScene () const {return mIsNotSolidScene;};
			void setNumberOfConservativeFrames (const unsigned int frames){mFrameConservativeVisibility = frames;}
			unsigned int getNumberOfConservativeFrames () const {return mFrameConservativeVisibility;}

			void setFirstDrawPending(const bool val){mFirstDrawPending = val;}
			bool getFirstDrawPending(){return mFirstDrawPending;}

			// mOcclusionModeIsHWOcclusion getter
			bool getOcclusionModeIsHWOcclusion() const { return mOcclusionModeIsHWOcclusion; }
			// mOcclusionModeIsHWOcclusion setter
			void setOcclusionModeIsHWOcclusion(bool val) { mOcclusionModeIsHWOcclusion = val; }

        private:


			//	don't query if no drawing done yet. (thus nothing can be hidden)
			bool mFirstDrawPending;
			// when wireframe testing
			bool mIsNotSolidScene;
            ///Keep a set of pre-allocated HW queries
            QuerySet mQueryPool;
            /// As it's initiated on first HW query use
            bool mIsQueryPoolNotInitiated;

            ///culling mode from frustum culling to coherent hierarchical Occlusion
	        unsigned int mMode; 

			/// number of frame we consider a node visible after a query set it visible once.
			unsigned int mFrameConservativeVisibility;
			/// Number of pixel upon which a node is considered as visible
	        unsigned int mVisibilityTreshold;   
            /// store number of nodes in Tree 	
	        unsigned int mNodeCount;

            /// store queried elements that didn't return results
            QueriedOcclusionElement mQueryQueue;

			/// if current occlusion mode is using HW occlusion queries
			bool mOcclusionModeIsHWOcclusion;
    };

}
#endif //__OctreeCHCOcclusion_H
