#ifndef __OctreeCHCOcclusionElement_H
#define __OctreeCHCOcclusionElement_H

#include "OgreHardwareOcclusionQuery.h"
#include "OgreRectangle2D.h"

namespace OgreOctreeCHC
{
    class OcclusionElement
    {
        public :
            OcclusionElement();
            virtual ~OcclusionElement();

            VisibilityData *getNodeData(const OctreeCHCCamera * const cam);
            void addCamNodeData (const OctreeCHCCamera * const cam);
            void removeCamNodeData (const OctreeCHCCamera * const cam);
         
			virtual bool isLeaf() const = 0;

            //virtual void _addToRenderQueue(Ogre::Camera* cam, Ogre::RenderQueue* const q, const bool onlyShadowCasters) = 0;
            //virtual void _addAlreadyNotifiedToVisibles(Ogre::Camera* cam, Ogre::RenderQueue* const q, const bool onlyShadowCasters) = 0;
            //virtual void notifyNodeObjects(Ogre::Camera*  cam) = 0;

			virtual void depthTraversal(DepthTraversal&) = 0;
	        virtual void depthTraversal(const DepthTraversalConst&) = 0;
			virtual void depthTraversal(const ConstDepthTraversalConst&) const = 0;

			virtual void breadthTraversal(BreadthTraversal&) = 0;
			virtual void breadthTraversal(const BreadthTraversalConst&) = 0;
			virtual void breadthTraversal(const ConstBreadthTraversalConst&) const = 0;
/*
			virtual const Ogre::Vector3        &getHalfSize() const = 0;               
*/
			Ogre::Rectangle2D *hasRectangle2d(){return mDebugRectangle2d;};
			Ogre::Rectangle2D *getRectangle2d();
                
            void setRegisteredtoCam(const bool flag)
			{
				mIsRegisteredToCam = flag;
			};

            virtual bool isOccluder () const = 0;

			inline bool isVisibilityResultReady()
			{ 
				octree_assert (query); 
				return query->isStillOutstanding();
			}
	public:
			// frustum culled
			bool							viewFrustumVisible;
			// Visible on screen, as certified by query result
			bool							queriedVisible;
			// query  associated to node
			Ogre::HardwareOcclusionQuery *	query;
			// Has been checked against renderer
			// (if not too far, or outside this render like shadows... )
			bool							notified;

			// Visibility Data for the current CHC loop. (cached to avoid map lookup.)
			VisibilityData					*currVis;

			inline bool hasOcclusionBoundingBox(){return (mOcclusionBoundingBox != 0);} 

			inline const Ogre::AxisAlignedBox &getBoundingBox() const
			{
				return getOcclusionBoundingBox()->getBoundingBox();
			}

			inline const Ogre::Sphere &getBoundingSphere() const
			{
				return getOcclusionBoundingBox()->getBoundingSphere();
			}

			inline OcclusionBoundingBox *getOcclusionBoundingBox() const
			{
				// Create an OcclusionBoundingBox if needed
// 				if(mOcclusionBoundingBox == 0)
// 				{
// 					mOcclusionBoundingBox = new OcclusionBoundingBox();
// 					mOcclusionBoundingBox->setupBoundingBox(*mCullBoundingBox);
// 				}
				octree_assert(mOcclusionBoundingBox);
				return mOcclusionBoundingBox;
			};

			void setOcclusionBoundingBox(const Ogre::AxisAlignedBox &box);
	       
			inline void setOcclusionParent(OcclusionElement * parent){mOcclusionParent = parent;};
			inline OcclusionElement *getOcclusionParent (){return mOcclusionParent;};  

        protected :

            /// if node has all camera information about occlusion
            bool							mIsRegisteredToCam;

        private :

			OcclusionElement *mOcclusionParent;  

            typedef std::map<unsigned int, VisibilityData* > NodeDataPerCamMap;
            typedef NodeDataPerCamMap::iterator              NodeDataPerCamMapiterator;
            NodeDataPerCamMap								 nodeDataPerCam; 
            
			Ogre::Rectangle2D		*mDebugRectangle2d;
            OcclusionBoundingBox	*mOcclusionBoundingBox;
            
    };

}//namespace OgreOctreeCHC

#endif //__OctreeCHCOcclusionElement_H
