#ifndef _OctreeCHCOcclusionSorterH
#define _OctreeCHCOcclusionSorterH

#include "OctreeCHCOcclusionElement.h"
#include "OctreeCHCOcclusionBoundingBox.h"

namespace OgreOctreeCHC
{
   
    //comparator for priority queue
    class FrontToBackNodeSorterOperator 
    {

    public:
		FrontToBackNodeSorterOperator(const Ogre::Vector3& position) :   
		  pos(position) 
	    {

	    };
		//-----------------------------------------------------------------------
	    inline bool operator()(OcclusionElement*& a, OcclusionElement*& b) const 
        {
            // could check if both distance == 0 then order visible first// ?
	        return vectorToBoxDistance (*a, pos) > vectorToBoxDistance (*b, pos);
		}       
		//-----------------------------------------------------------------------
		inline bool operator()(const OcclusionElement*& a, const OcclusionElement*& b) const 
		{
			// could check if both distance == 0 then order visible first ?
			return vectorToBoxDistance (*a, pos) > vectorToBoxDistance (*b, pos);
		}       
		//-----------------------------------------------------------------------
        inline Ogre::Real vectorToBoxDistance(const OcclusionElement &a, const Ogre::Vector3& point)  const
        {
			const Ogre::AxisAlignedBox &box (a.getBoundingBox());
            //const Ogre::Vector3 &boxMin (box.getMinimum ());
			const Ogre::Vector3 halfSize ((box.getMaximum () - box.getMinimum ()) * Ogre::Real(0.5));
            // work in the box's coordinate system
            const Ogre::Vector3 kDiff (point - (halfSize + box.getMinimum ()));
            // compute squared distance and closest point on box
            Ogre::Real fSqrDistance (Ogre::Real(0.0));
            for (unsigned int i = 0; i < 3; i++)
            {
                const Ogre::Real kClosest (kDiff[i]);
                const Ogre::Real khalfSize (halfSize[i]);
                if (kClosest < -khalfSize)
                {
                    const Ogre::Real fDelta (kClosest + khalfSize);
                    fSqrDistance += fDelta * fDelta;
                }
                else if (kClosest > khalfSize)
                {
                    const Ogre::Real fDelta (kClosest - khalfSize);
                    fSqrDistance += fDelta * fDelta;
                }
            }
            return fSqrDistance;
        }

    private:
	    const Ogre::Vector3 pos;
    };
}//namespace OgreOctreeCHC

#endif //_OctreeCHCOcclusionSorterH
