#ifndef __OctreeCHCOcclusionDepthTraversal_H
#define __OctreeCHCOcclusionDepthTraversal_H

#include "OctreeCHCPrerequisites.h"


namespace OgreOctreeCHC
{
    //-----------------------------------------------------------------------
    class ConstDepthTraversalConst 
    {

        public:
		    virtual void onDepthTree(const SpatialTree& n) const = 0;
	        virtual void onDepthLeaf(const SpatialTreeNode&) const = 0;

        protected:
			virtual void depthTraverseChildren(const SpatialTree& n) const;
    };
    //-----------------------------------------------------------------------
    class DepthTraversalConst 
    {

        public:
		    virtual void onDepthTree(SpatialTree& n) const = 0;
	        virtual void onDepthLeaf(SpatialTreeNode&) const = 0;

        protected:
			virtual void depthTraverseChildren(SpatialTree& n) const ;
    };
    //-----------------------------------------------------------------------
    class DepthTraversal 
    {

        public:
		    virtual void onDepthTree(SpatialTree& n) = 0;
	        virtual void onDepthLeaf(SpatialTreeNode&) = 0;

        protected:
			virtual void depthTraverseChildren(SpatialTree& n);
	};
	//-----------------------------------------------------------------------
	class ConstBreadthTraversalConst 
	{

	public:
		virtual void onBreadthTree(const SpatialTree& n) const = 0;
		virtual void onBreadthLeaf(const SpatialTreeNode&) const = 0;

	protected:
		virtual void breadthTraverseChildren(const SpatialTree& n) const;
	};
	//-----------------------------------------------------------------------
	class BreadthTraversalConst 
	{

	public:
		virtual void onBreadthTree(SpatialTree& n) const = 0;
		virtual void onBreadthLeaf(SpatialTreeNode&) const = 0;

	protected:
		virtual void breadthTraverseChildren(SpatialTree& n) const ;
	};
	//-----------------------------------------------------------------------
	class BreadthTraversal 
	{

	public:
		virtual void onBreadthTree(SpatialTree& n) = 0;
		virtual void onBreadthLeaf(SpatialTreeNode&) = 0;

	protected:
		virtual void breadthTraverseChildren(SpatialTree& n);
	};

	//-----------------------------------------------------------------------
	//-----------------------------------------------------------------------
}

#include "OctreeCHCOcclusionElement.h"
#include "OgreSpatialTree.h"
#include "OgreSpatialTreeNode.h"

namespace OgreOctreeCHC
{
	//-----------------------------------------------------------------------
	inline void ConstDepthTraversalConst::depthTraverseChildren(const SpatialTree& node) const 
	{                
		if (!node.mNodes.empty())
		{
			//Add stuff to be rendered;
			SpatialTreeNodeList::const_iterator it = node.mNodes.begin();
			const SpatialTreeNodeList::const_iterator itEnd = node.mNodes.end();
			while (it != itEnd)
			{
				(*it)->depthTraversal (*this);
				++it;
			}
		}

		for (unsigned int i = 0; i < node.getChildrenMaxCount(); i++)
		{
			const SpatialTree * const n = node.mChildren[i];
			if (n && n->hasChildren())
				n->depthTraversal (*this);
		}

	}
	//-----------------------------------------------------------------------
	inline void DepthTraversalConst::depthTraverseChildren(SpatialTree& node) const 
	{                  
		if (!node.mNodes.empty())
		{
			//Add stuff to be rendered;
			SpatialTreeNodeList::iterator it = node.mNodes.begin();
			const SpatialTreeNodeList::iterator itEnd = node.mNodes.end();
			while (it != itEnd)
			{
				(*it)->depthTraversal(*this);
				++it;
			}            
		}

		for (unsigned int i = 0; i < node.getChildrenMaxCount(); i++)
		{
			SpatialTree * const n = node.mChildren[i];
			if (n && n->hasChildren())
				n->depthTraversal (*this);
		}
	} 
	//-----------------------------------------------------------------------
	inline void DepthTraversal::depthTraverseChildren(SpatialTree& node)
	{                  
		// hasLocalChildren
		if (!node.mNodes.empty())
		{
			//Add stuff to be rendered;
			SpatialTreeNodeList::iterator it = node.mNodes.begin();
			const SpatialTreeNodeList::iterator itEnd = node.mNodes.end();
			while (it != itEnd)
			{
				(*it)->depthTraversal (*this);
				++it;
			}
		}

		for (unsigned int i = 0; i < node.getChildrenMaxCount(); i++)
		{
			SpatialTree * const n = node.mChildren[i];
			if (n && n->hasChildren())
				n->depthTraversal (*this);
		}
	}
	//-----------------------------------------------------------------------
	inline void ConstBreadthTraversalConst::breadthTraverseChildren(const SpatialTree& node) const 
	{                
		if (!node.mNodes.empty())
		{
			//Add stuff to be rendered;
			SpatialTreeNodeList::const_iterator it = node.mNodes.begin();
			const SpatialTreeNodeList::const_iterator itEnd = node.mNodes.end();
			while (it != itEnd)
			{
				(*it)->breadthTraversal (*this);
				++it;
			}
		}

		for (unsigned int i = 0; i < node.getChildrenMaxCount(); i++)
		{
			const SpatialTree * const n = node.mChildren[i];
			if (n && n->hasChildren())
				n->breadthTraversal (*this);
		}

	}
	//-----------------------------------------------------------------------
	inline void BreadthTraversalConst::breadthTraverseChildren(SpatialTree& node) const 
	{                  
		if (!node.mNodes.empty())
		{
			//Add stuff to be rendered;
			SpatialTreeNodeList::iterator it = node.mNodes.begin();
			const SpatialTreeNodeList::iterator itEnd = node.mNodes.end();
			while (it != itEnd)
			{
				(*it)->breadthTraversal(*this);
				++it;
			}            
		}

		for (unsigned int i = 0; i < node.getChildrenMaxCount(); i++)
		{
			SpatialTree * const n = node.mChildren[i];
			if (n && n->hasChildren())
				n->breadthTraversal (*this);
		}
	} 
	//-----------------------------------------------------------------------
	inline void BreadthTraversal::breadthTraverseChildren(SpatialTree& node)
	{                  
		// hasLocalChildren
		if (!node.mNodes.empty())
		{
			//Add stuff to be rendered;
			SpatialTreeNodeList::iterator it = node.mNodes.begin();
			const SpatialTreeNodeList::iterator itEnd = node.mNodes.end();
			while (it != itEnd)
			{
				(*it)->breadthTraversal (*this);
				++it;
			}
		}

		for (unsigned int i = 0; i < node.getChildrenMaxCount(); i++)
		{
			SpatialTree * const n = node.mChildren[i];
			if (n && n->hasChildren())
				n->breadthTraversal (*this);
		}
	}

}
#endif //__OctreeCHCOcclusionDepthTraversal_H
