/***************************************************************************
  OctreeCHCPrerequisites.h  -  description
  -------------------
	begin                : Sun Oct 26 2003
	copyright            : (C) 2003-2006 by Jose A Milan && Tuan Kuranes
	email                : spoke2@supercable.es && tuan.kuranes@free.fr
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __OCTREECHCPrerequisites_H__
#define __OCTREECHCPrerequisites_H__

//-----------------------------------------------------------------------
// Options 
//-----------------------------------------------------------------------

//#define _VISIBILITYDEBUG


#include "OgrePrerequisites.h"



//-----------------------------------------------------------------------
// Forward declarations
//-----------------------------------------------------------------------

#include <vector>
#include <map>
#include <queue>
#include <list>
#include <stack>
#include <limits>


namespace OgreOctreeCHC
{

	class RadarFrustum;

    // Octree Scene management
	
	typedef std::list < Ogre::MovableObject* > MovableObjectList;
    typedef std::list < Ogre::Renderable* > RenderableList;
    typedef std::list < Ogre::SceneNode* > NodeList;
    typedef std::list < Ogre::SceneNode* > SceneNodeList;

    class OctreeCHCSceneManager;


	// Any tree implementation spatial algorithms handler should derive from that
	class SpatialTreeManager;
	// Any tree implementation should derive from that
	class SpatialTree;
	// Any Scene Node implementation should derive from that
    class SpatialTreeNode;
	typedef std::list < SpatialTreeNode* > SpatialTreeNodeList;


    class OctreeCHCManager;
    class OctreeCHC;
	typedef std::deque <OctreeCHC*> OctreeList;
    class OctreeCHCNode;
    typedef std::list<OctreeCHCNode*> OctreeCHCNodeList;

    // hardware occlusion management
    class QuerySet;
    class OcclusionCulling;
    class VisibilityData;
    class OcclusionElement;


    class OctreeCHCCamera;


    typedef enum culling_modes
    { 
        // STANDARD_WALK = 0,
        VIEW_FRUSTUM = 0,
		VIEW_FRUSTUM_MASK,
		VIEW_FRUSTUM_MASK_TEMPORAL,
		VIEW_FRUSTUM_DIRECT,
		VIEW_FRUSTUM_RADAR,
		VIEW_FRUSTUM_COMPLEX,
        STOP_AND_WAIT,
        CHC, 
        CHC_CONSERVATIVE, 
        NUM_CULLING_MODE
    };


    class OcclusionBoundingBox;
    typedef std::list<OcclusionElement*> OcclusionElementList;
    typedef std::list<OctreeCHCNode*> OctreeCHCNodeList;

    class FrontToBackNodeSorterOperator;
    //priority queue for the nodes to depthTraversal
	typedef std::priority_queue<OcclusionElement*,std::vector<OcclusionElement*>,FrontToBackNodeSorterOperator> FrontToBackNodeSorterPriorityQueue;
	//queue for the nodes with running occlusion queries
	typedef std::queue<OcclusionElement*> QueriedOcclusionElement;  //queue for the nodes with running occlusion queries
    	   

    class DepthTraversal;
    class DepthTraversalConst;
    class ConstDepthTraversalConst;

	class BreadthTraversal;
	class BreadthTraversalConst;
	class ConstBreadthTraversalConst;

    class CHCTraversal;
    class StopAndWaitTraversal;
    class ViewFrustumCullingDepthTraversal;
    class ViewFrustumCullingDepthTraversalDirect;
    class TreeOverlayDebug;

}
//-----------------------------------------------------------------------
// Windows Settings
//-----------------------------------------------------------------------

#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32 
#   ifdef PLUGIN_OCTREECHC_EXPORTS
#       define _OctreeCHCExport __declspec(dllexport) 
#   else 
#       define _OctreeCHCExport __declspec(dllimport) 
#   endif 
#else 
#   define _OctreeCHCExport 
#endif 

//STL trick to call delete in a for_each
struct delete_object
{
    template <typename T>
    void operator()(T *ptr){ delete ptr; ptr = 0;}
};


#ifndef ogre_restrict
	#ifdef __INTEL_COMPILER
		#define ogre_restrict restrict
	#elif _MSC_VER >= 1400
		#define ogre_restrict __restrict
	#else
		#define ogre_restrict
	#endif
#endif


// DEBUGGING

#ifdef _DEBUG
	#define NEED__OCTREE_CHC_ASSERT
#endif

#ifdef _OCTREE_CHC_RELEASE_ASSERT
	#define NEED__OCTREE_CHC_ASSERT
#endif

#if defined(NEED__OCTREE_CHC_ASSERT)
	//#define assert(exp) (void)((exp) || (_assert(#exp, __FILE__, __LINE__), 0))
	#define _octree_assert(exp) {if(!(exp)) _asm{int 0x03}}
	
	#define octree_assert(exp) _octree_assert(exp)  
#else 
	#define  octree_assert(exp) 
#endif // DEBUGGING

#endif //__OCTREECHCPrerequisites_H__
