#ifndef __OctreeCHCQueryDepthTraversal_H
#define __OctreeCHCQueryDepthTraversal_H

#include "OctreeCHCPrerequisites.h"
#include "OctreeCHCCamera.h"

#include "OctreeCHCOcclusionTraversal.h"

namespace OgreOctreeCHC
{
    //-----------------------------------------------------------------------
	class SphereQueryDepthTraversal : public DepthTraversal 
    {
	    public:

			SphereQueryDepthTraversal (
				const Ogre::Sphere &s, 
				std::list < Ogre::SceneNode * > &list, 
				const Ogre::SceneNode* const exclude) :
				mSphere(s),
				mList(list),
				mExclude(exclude),
				mIsVisible (false)
			{
			}

            void onDepthTree(SpatialTree&);
            void onDepthLeaf(SpatialTreeNode&);

	    private:
			const Ogre::Sphere &mSphere;
			std::list < Ogre::SceneNode * > &mList;
			const Ogre::SceneNode* const mExclude;
			bool mIsVisible;

	};
  
    //-----------------------------------------------------------------------
	class PlaneBoundedVolumeQueryDepthTraversal : public DepthTraversal 
    {

	    public:
			PlaneBoundedVolumeQueryDepthTraversal (
				const Ogre::PlaneBoundedVolume &p, 
				std::list < Ogre::SceneNode * > &list, 
				const Ogre::SceneNode* const exclude) :
				mPlaneBoundedVolume(p),
				mList(list),
				mExclude(exclude),
				mIsVisible (false)
			{
			}

            void onDepthTree(SpatialTree&);
            void onDepthLeaf(SpatialTreeNode&);

		private:
			const Ogre::PlaneBoundedVolume &mPlaneBoundedVolume;
			std::list < Ogre::SceneNode * > &mList;
			const Ogre::SceneNode* const mExclude;
			bool mIsVisible;
	};
	//-----------------------------------------------------------------------
	class RayQueryDepthTraversal : public DepthTraversal 
	{

	public:

		RayQueryDepthTraversal (const Ogre::Ray &r, 
			std::list < Ogre::SceneNode * > &list, 
			const Ogre::SceneNode* const exclude) :
			mRay(r),
			mList(list),
			mExclude(exclude),
			mIsVisible (false)
		{
		}

		void onDepthTree(SpatialTree&);
		void onDepthLeaf(SpatialTreeNode&);

	private:
		const Ogre::Ray &mRay;
		std::list < Ogre::SceneNode * > &mList;
		const Ogre::SceneNode* const mExclude;
		bool mIsVisible;

	};
	//-----------------------------------------------------------------------
	class BoxQueryDepthTraversal : public DepthTraversal 
	{

	public:

		BoxQueryDepthTraversal (const Ogre::AxisAlignedBox &b, 
			std::list < Ogre::SceneNode * > &list, 
			const Ogre::SceneNode* const exclude):
			mBoundingBox(b),
			mList(list),
			mExclude(exclude),
			mIsVisible (false)
		{

		}

		void onDepthTree(SpatialTree&);
		void onDepthLeaf(SpatialTreeNode&);

	private:
		const Ogre::AxisAlignedBox &mBoundingBox;
		std::list < Ogre::SceneNode * > &mList;
		const Ogre::SceneNode* const mExclude;
		bool mIsVisible;

	};
}
#endif //__OctreeCHCQueryDepthTraversal_H
