/*
-----------------------------------------------------------------------------
This source file is part of OGRE
(Object-oriented Graphics Rendering Engine)
For the latest info, see http://www.ogre3d.org/

Copyright (c) 2000-2006 The OGRE Team
Also see acknowledgements in Readme.html

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

#ifndef __RADAR_FRUSTUM_
#define __RADAR_FRUSTUM_

#include "OgreQuaternion.h"
#include "OgreVector3.h"
namespace OgreOctreeCHC
{

	class RadarFrustum  
	{
	public:
		RadarFrustum();

		void SetPerspective( const Ogre::Radian FOV, const Ogre::Real ViewAspect, const Ogre::Real NearZ, const Ogre::Real FarZ );
		void Build( const Ogre::Vector3& Eye, const Ogre::Vector3& Forward, const Ogre::Vector3& Right, const Ogre::Vector3& Up );

		bool IsPointIn ( const Ogre::Vector3& Point ) const;
		bool IsHullIn  ( const Ogre::Vector3* aVertices, const char nVertices ) const;
		bool IsSphereIn( const Ogre::Vector3& Center, const float Radius ) const;
		bool IsAABBIn  ( const Ogre::Vector3& v1, const Ogre::Vector3& v2) const;
		bool IsOBBIn   ( const Ogre::Vector3& Center, const Ogre::Vector3& HalfDimensions, const Ogre::Quaternion& RotationQuat) const;

	private:


		Ogre::Vector3 mEyePosition,
			mForwardVector,
			mRightVector,
			mUpVector;

		Ogre::Real mRFactor, 
			mUFactor;

		Ogre::Real mNearZ, 
			mFarZ;
	};

}
#endif __RADAR_FRUSTUM_
