/*
-----------------------------------------------------------------------------
This source file is part of OGRE
(Object-oriented Graphics Rendering Engine)
For the latest info, see http://www.ogre3d.org/

Copyright (c) 2000-2006 The OGRE Team
Also see acknowledgements in Readme.html

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/
/***************************************************************************
OctreeCHCnode.cpp  -  description
-------------------
begin                : Fri Sep 27 2002
copyright            : (C) 2002 by Jon Anderson
email                : janders@users.sf.net

Enhancements 2003 - 2004 (C) The OGRE Team

***************************************************************************/

#include "OctreeCHCPrecompiledHeaders.h"

#include "OgreRoot.h"

#include "OctreeCHCNode.h"
#include "OctreeCHCOcclusionBoundingBox.h"
#include "OgreWireBoundingBox.h"
#include "OgreRenderQueue.h"


using namespace Ogre;

namespace OgreOctreeCHC
{   
    //-----------------------------------------------------------------------
    OctreeCHCNode::OctreeCHCNode(Ogre::SceneManager* creator) : 
        SpatialTreeNode(creator)
    {       
    }
    //-----------------------------------------------------------------------
    OctreeCHCNode::OctreeCHCNode(Ogre::SceneManager* creator, const Ogre::String& name) : 
        SpatialTreeNode(creator, name)
    {
    }
    //-----------------------------------------------------------------------
    OctreeCHCNode::~OctreeCHCNode()
	{
    }
    //-----------------------------------------------------------------------
    //same as SceneNode, only it doesn't care about children...
    void OctreeCHCNode::_updateBounds(void)
    {
		SpatialTreeNode::_updateBounds();
		mHalfSize = (mWorldAABB.getMaximum () - mWorldAABB.getMinimum ()) * 0.5; 
		setOcclusionBoundingBox(mWorldAABB);
    }
    //-----------------------------------------------------------------------
    /** Since we are loose, only check the center.
    */
    bool OctreeCHCNode::_isIn(const Ogre::AxisAlignedBox &box) const
    {
	    // Always fail if not in the scene graph or box is null
		if (!mIsInSceneGraph || box.isNull()) 
			return false;
		// Always succeed if AABB is infinite
		if (box.isInfinite())
			return true;

        // Object Bounding Box Center
        const Ogre::Vector3 center = mWorldAABB.getMaximum().midPoint(mWorldAABB.getMinimum());

        // Min and Max of Octree BBox
        const Ogre::Vector3 &bmin (box.getMinimum());
        const Ogre::Vector3 &bmax (box.getMaximum());

        // Object Bbox center is IN Octree BBox ?
        const bool centre  = bmax.x >= center.x && bmax.y >= center.y && bmax.z >= center.z &&
                             bmin.x <= center.x && bmin.y <= center.y && bmin.z <= center.z;
        if (!centre)
          	return false;
        
         // Even if covering the centre line, need to make sure this BB is not large
        // enough to require being moved up into parent. When added, bboxes would
        // end up in parent due to cascade but when updating need to deal with
        // bbox growing too large for this child
        const Ogre::Vector3 octreeSize (bmax - bmin);
        const Ogre::Vector3 nodeSize (mWorldAABB.getMaximum() - mWorldAABB.getMinimum());

        return (nodeSize < octreeSize);

    }
}
