
#include "OctreeCHCPrecompiledHeaders.h"

#include "OctreeCHCOcclusionTraversal.h"

#include "OctreeCHCCamera.h"
#include "OctreeCHC.h"
#include "OctreeCHCNode.h"
#include "OctreeCHCOcclusionElement.h"
#include "OctreeCHCOcclusion.h"

#include "OctreeCHCOcclusionTraversal.h"
#include "OctreeCHCOcclusionCameraTraversal.h"

#include "OctreeCHCOcclusionVisibilityData.h"

using namespace Ogre;

namespace OgreOctreeCHC
{
	//-----------------------------------------------------------------------
    //-----------------------------------------------------------------------
    void RegisterCameraDepthTraversal::onDepthTree(SpatialTree& n) const 
    {
        n.addCamNodeData (cam);
        depthTraverseChildren (n);
    }
    //-----------------------------------------------------------------------
    void RegisterCameraDepthTraversal::onDepthLeaf (SpatialTreeNode& n) const 
    {
        n.addCamNodeData (cam);
	}
	//-----------------------------------------------------------------------
    //-----------------------------------------------------------------------
    void UnregisterCameraDepthTraversal::onDepthTree(SpatialTree& n) const 
    {
        n.removeCamNodeData (cam);
        depthTraverseChildren (n);
    }
    //-----------------------------------------------------------------------
    void UnregisterCameraDepthTraversal::onDepthLeaf (SpatialTreeNode& n) const 
    {
        n.removeCamNodeData (cam);
	} 
	//-----------------------------------------------------------------------
	//-----------------------------------------------------------------------
	void ResetCameraVisibilityInfo::onDepthTree(SpatialTree& n) const 
	{
		n.queriedVisible = false; 

		VisibilityData * const data = n.getNodeData(cam);
		data->frameID = 0;
		data->wasQueriedVisible = false;
		data->lastQueryFrameId = 0;
		depthTraverseChildren (n);
		
		if (!n.mNodes.empty())
		{
			//Add stuff to be rendered;
			SpatialTreeNodeList::iterator it = n.mNodes.begin();
			const SpatialTreeNodeList::iterator itEnd = n.mNodes.end();
			while (it != itEnd)
			{
				(*it)->setOcclusionParent(&n);
				++it;
			}            
		}

		for (unsigned int i = 0; i < n.getChildrenMaxCount(); i++)
		{
			SpatialTree * const nT = n.mChildren[i];
			if (nT)
				nT->setOcclusionParent(&n);
		}
	}
	//-----------------------------------------------------------------------
	void ResetCameraVisibilityInfo::onDepthLeaf (SpatialTreeNode& n) const 
	{
		n.queriedVisible = false; 

		VisibilityData * const data = n.getNodeData(cam);
		data->frameID = 0;
		data->wasQueriedVisible = false;
		data->lastQueryFrameId = 0;
	} 
	
}