

#include "OctreeCHCPrecompiledHeaders.h"

  

#include <stack>
#include <queue>

#include "OgreRectangle2D.h"

#include "OctreeCHCCamera.h"
#include "OctreeCHCOcclusionVisibilityData.h"
#include "OctreeCHCOcclusionElement.h"
#include "OctreeCHCOcclusionBoundingBox.h"


using namespace Ogre;

namespace OgreOctreeCHC
{
    //---------------------------------------------------------------------
    OcclusionElement::OcclusionElement() :
    	mDebugRectangle2d(0),
        mIsRegisteredToCam (false),
		query(0),
		mOcclusionParent(0),
		mOcclusionBoundingBox(0),
		currVis(0)
    {
    } 
    //---------------------------------------------------------------------
    OcclusionElement::~OcclusionElement() 
    {    	
        delete mDebugRectangle2d;          
        delete mOcclusionBoundingBox;
        NodeDataPerCamMapiterator i = nodeDataPerCam.begin ();
        while (i != nodeDataPerCam.end())
        {
            delete i->second;
            ++i;
        }
        nodeDataPerCam.clear();
    }  
    //---------------------------------------------------------------------
    VisibilityData *OcclusionElement::getNodeData(const OctreeCHCCamera * const cam)
    {
		//octree_assert (nodeDataPerCam.find(cam->getId()) != nodeDataPerCam.end());
		//return nodeDataPerCam[cam->getId()];
		NodeDataPerCamMapiterator it = nodeDataPerCam.find(cam->getRenderId());
		if (it != nodeDataPerCam.end())
		{
			return it->second;
		}
		else
		{
			VisibilityData *vis = new VisibilityData();
			nodeDataPerCam[cam->getRenderId()] = vis;
			return vis;
		}
    }
    //---------------------------------------------------------------------
    void OcclusionElement::addCamNodeData (const OctreeCHCCamera * const cam)
    {
        if (nodeDataPerCam.find(cam->getRenderId()) == nodeDataPerCam.end())
			nodeDataPerCam[cam->getRenderId()] = new VisibilityData();
    }
    //---------------------------------------------------------------------
    void OcclusionElement::removeCamNodeData (const OctreeCHCCamera * const cam)
    {
        NodeDataPerCamMapiterator i = nodeDataPerCam.find (cam->getRenderId());
        if (i != nodeDataPerCam.end())
		{
			delete i->second;
			nodeDataPerCam.erase (i);
		}
    }
    
    //---------------------------------------------------------------------
	Rectangle2D *OcclusionElement::getRectangle2d()
    {
        if (!mDebugRectangle2d)
        {
			mDebugRectangle2d = new Rectangle2D(false);

			mDebugRectangle2d->setRenderQueueGroup(RENDER_QUEUE_OVERLAY);

			mDebugRectangle2d->setUseIdentityProjection(true);
			mDebugRectangle2d->setUseIdentityView(true);
			mDebugRectangle2d->setPolygonModeOverrideable(false);
        }
        return mDebugRectangle2d;
    }
    //---------------------------------------------------------------------
	 void OcclusionElement::setOcclusionBoundingBox(const Ogre::AxisAlignedBox &box) 
	 { 
		mOcclusionBoundingBox = new OcclusionBoundingBox();
		mOcclusionBoundingBox->setupBoundingBox(box);
	 }; 
}//namespace OgreOctreeCHC
