
#include "OctreeCHCPrecompiledHeaders.h"


#include "OctreeCHCCamera.h"
#include "OctreeCHC.h"
#include "OctreeCHCNode.h"

#include "OctreeCHCOcclusionVisibilityData.h"
#include "OctreeCHCOcclusionElement.h"
#include "OctreeCHCOcclusionSorter.h"
#include "OctreeCHCOcclusion.h"

#include "OctreeCHCOcclusionTraversal.h"
#include "OctreeCHCOcclusionSWTraversal.h"

using namespace Ogre;

namespace OgreOctreeCHC
{
    //-----------------------------------------------------------------------
	StopAndWaitTraversal::StopAndWaitTraversal  (OcclusionCulling& o, Ogre::VisibleObjectsBoundsInfo * const visibleBounds):
        mOcclusion(o), 
        mIsVisible (OctreeCHCCamera::NONE),
        mCamera(mOcclusion.mCurrentCam),
		mVisibleBounds(visibleBounds),
		mStack (FrontToBackNodeSorterPriorityQueue(FrontToBackNodeSorterOperator (mOcclusion.mCurrentCam->getDerivedPosition ())))
    {
        
	}
	//-----------------------------------------------------------------------
	bool StopAndWaitTraversal::isVisible(OcclusionElement & n) 
	{
		// issue query
		if (n.isLeaf () && n.isOccluder())
		{
			VisibilityData * const vis =  n.getNodeData (mCamera);
			n.currVis = vis;
			// As it's an Occluder Octree Node Real draw is visible or hidden by previous draws
			if (mOcclusion.issueDrawQuery (static_cast <OctreeCHCNode &> (n)))
			{
				// we just wait for result
				return mOcclusion.isQueryResultIsVisible (n);
			}
			else
			{
				return false;
			}
		}
		else
		{ 
			// test if Octree BBox Real draw is visible or hidden by previous draws
			if (mOcclusion.issueBboxQuery (n))
			{
				// we just wait for result
				return mOcclusion.isQueryResultIsVisible (n);
			}
			else
			{
				// Special case where we didn't draw anything yet anyway.
				return true;
			}
		}

	}
	//-----------------------------------------------------------------------
	void StopAndWaitTraversal::onBreadthTree(SpatialTree &n)
	{
		mStack.push(&n);
	}
	//-----------------------------------------------------------------------
	void StopAndWaitTraversal::onBreadthLeaf(SpatialTreeNode& n)
	{
		mStack.push(&n);
	}
    //-----------------------------------------------------------------------
    void StopAndWaitTraversal::onDepthLeaf (SpatialTreeNode &n)
    {
		if(mOcclusion.mCurrentCam->isVisible (n._getWorldAABB()))
        {
            mOcclusion.mTraversedNodesCount++;
			n.viewFrustumVisible = true;
			// test if Octree Node is visible or hidden by previous draws
            const bool isitVisible = isVisible (n);
		    if (isitVisible) 
            {
			    mOcclusion.mObjectCount++;
                mOcclusion.queueDraw (n);
            } 
		     n.queriedVisible = isitVisible;
        }
        else 
        {
            mOcclusion.mTraversedNodesCount++;
            n.viewFrustumVisible = false;
	        mOcclusion.mFrustumCulledNodesCount++;
        }
		    
    }
    //-----------------------------------------------------------------------
    void StopAndWaitTraversal::onDepthTree(SpatialTree & n)
    {
        if(mOcclusion.mCurrentCam->isVisible(n._getCullBounds ())) 
        {
            mOcclusion.mTraversedNodesCount++; 
            n.viewFrustumVisible = true;
			// test if Octree BBOX is visible or hidden by previous draws
            const bool isitVisible = isVisible(n);
		    if (isitVisible) 
			{
				// fill stack  with all sub depth child
				breadthTraverseChildren (n);

				//traverse nearer child first
				while (!mStack.empty ())
				{
					OcclusionElement& nstacked = *mStack.top ();
					mStack.pop ();
					nstacked.depthTraversal(*this);
				}
			}

		    n.queriedVisible = isitVisible;
	    }
		else 
        {
            mOcclusion.mTraversedNodesCount++; 
            n.viewFrustumVisible = false;
	        mOcclusion.mFrustumCulledNodesCount++;
        }
    }
}